package com.sample;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import junit.framework.TestCase;

import org.drools.core.time.TimerService;
import org.jbpm.process.audit.JPAAuditLogService;
import org.jbpm.process.core.timer.TimerServiceRegistry;
import org.jbpm.runtime.manager.impl.RuntimeEnvironmentBuilder;
import org.jbpm.services.task.identity.JBossUserGroupCallbackImpl;
import org.jbpm.test.JBPMHelper;
import org.junit.Test;
import org.kie.api.io.ResourceType;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.manager.RuntimeEngine;
import org.kie.api.runtime.manager.RuntimeEnvironment;
import org.kie.api.runtime.manager.RuntimeManager;
import org.kie.api.runtime.manager.RuntimeManagerFactory;
import org.kie.api.runtime.process.ProcessInstance;
import org.kie.api.task.TaskService;
import org.kie.api.task.UserGroupCallback;
import org.kie.api.task.model.TaskSummary;
import org.kie.internal.io.ResourceFactory;
import org.kie.internal.runtime.manager.context.EmptyContext;

import bitronix.tm.BitronixTransactionManager;
import bitronix.tm.TransactionManagerServices;
import bitronix.tm.resource.jdbc.PoolingDataSource;

/**
 * This is a sample file to launch a process.
 */
public class ProcessJPATest extends TestCase {

    private static EntityManagerFactory emf;

    @Test
    public void testProcess() throws Exception {
        
        try {

            setup();

            System.out.println("--------------- 1st run");
            
            RuntimeManager manager = getRuntimeManager("BPMN2-ExceptionAfterTimer.bpmn2");
            RuntimeEngine runtime = manager.getRuntimeEngine(EmptyContext.get());
            KieSession ksession = runtime.getKieSession();

            ProcessInstance pi = ksession.startProcess("com.bpms.customer.RuntimeExceptionAfterTimer");
            try {
                Thread.sleep(3000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            
            pi = ksession.getProcessInstance(pi.getId());
            assertNull(pi);

            // -----------
            manager.disposeRuntimeEngine(runtime);
            manager.close();
            
            System.out.println("--------------- 2nd run");
            
            RuntimeManager manager2 = getRuntimeManager("BPMN2-ExceptionAfterTimer.bpmn2"); // This pushes 'null' jobHandle into jobHandles via org.jbpm.process.instance.timer.TimerManager$ProcessTimerInputMarshaller.deserialize()
            RuntimeEngine runtime2 = manager2.getRuntimeEngine(EmptyContext.get());
            KieSession ksession2 = runtime2.getKieSession();
            ksession2.insert("ZZZ"); // Now commit causes NPE in org.jbpm.process.core.timer.impl.GlobalTimerService.getTimerJobInstances()
            
            // This can also cause NPE directly
//            TimerService timerService = TimerServiceRegistry.getInstance().get(manager2.getIdentifier() +  TimerServiceRegistry.TIMER_SERVICE_SUFFIX);
//            timerService.getTimerJobInstances(ksession2.getId());

        } catch (Throwable th) {
            th.printStackTrace();
        }

        System.exit(0);
    }
    
    private static void setup() {
        // for H2 datasource
        JBPMHelper.startH2Server();
        JBPMHelper.setupDataSource();

        // for external database datasource
//         setupDataSource();
        
        Map configOverrides = new HashMap();
        configOverrides.put("hibernate.hbm2ddl.auto", "create"); // Uncomment if you don't want to clean up tables
        configOverrides.put("hibernate.dialect", "org.hibernate.dialect.H2Dialect"); // Edit for other databases
//        configOverrides.put("hibernate.dialect", "org.hibernate.dialect.MySQL5InnoDBDialect");

        emf = Persistence.createEntityManagerFactory("org.jbpm.persistence.jpa", configOverrides);

    }

    private static RuntimeManager getRuntimeManager(String process) {

        Properties properties = new Properties();
        properties.setProperty("krisv", "");
        properties.setProperty("mary", "");
        properties.setProperty("john", "");
        UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImpl(properties);


        RuntimeEnvironment environment =
                RuntimeEnvironmentBuilder.getDefault()
                .persistence(true)
                .entityManagerFactory(emf)
                .userGroupCallback(userGroupCallback)
                .addAsset(ResourceFactory.newClassPathResource(process), ResourceType.BPMN2)
                .get();
//        return RuntimeManagerFactory.Factory.get().newPerProcessInstanceRuntimeManager(environment);
        return RuntimeManagerFactory.Factory.get().newSingletonRuntimeManager(environment);


    }

//    public static PoolingDataSource setupDataSource() {
//        // Please edit here when you want to use your database
//        PoolingDataSource pds = new PoolingDataSource();
//        pds.setUniqueName("jdbc/jbpm-ds");
//        pds.setClassName("bitronix.tm.resource.jdbc.lrc.LrcXADataSource");
//        pds.setMaxPoolSize(5);
//        pds.setAllowLocalTransactions(true);
//        pds.getDriverProperties().put("user", "mysql");
//        pds.getDriverProperties().put("password", "mysql");
//        pds.getDriverProperties().put("url", "jdbc:mysql://localhost:3306/testbpms603");
//        pds.getDriverProperties().put("driverClassName", "com.mysql.jdbc.Driver");
//        pds.init();
//        return pds;
//    }
}